package translator;

import java.util.ArrayList;
import java.util.HashMap;

import se.Action;
import se.Message;

/**
 * Class representing an entity.
 * 
 * @author Fabio Pettenuzzo
 * @author Marco Palamà
 * @version 1.0, 12/13 
 */
public class Entity {
	
	/** Parent entity of this entity */
	private Entity parent=null;
	/** Maps a symbol name to the object representing it. */
	private HashMap<String, Message> symbols;
	/** Collection of actions of this entity. */
	private ArrayList<Action> actions;
	/** Collection of temporary actions of this entity. */
	private ArrayList<Action> tempActions;
	/** Collection of temporary goal actions of this entity. */
	private ArrayList<Action> goalTempActions;
	/** Entity name. */
	private String name;
	/** Entity formal parameters. */
	private ArrayList<String> formalParams;
	/** Entity actual parameters. */
	private ArrayList<Message> actualParams; 
	/** Collection of goal actions */
	private ArrayList<Action> entityGoals;
	/** 
	 * Maps a goal action to an object containing:
	 * The sender in position 0.
	 * The receiver in position 1.
	 * The goal label in position 2.
	 * The goal argument in position 3.
	 */
	private HashMap<Action, Object[]> goalParameter; // sender, receiver, stringa goal, argomento
	private Message sessionActor;
	/** Temporary sender */
	private Message tempSender;
	/** Temporary receiver */
	private Message tempReceiver;
	
	/**
	 * Constructor of the Entity class.
	 * @param parent The parent entity
	 * @param name The entity name
	 */
	Entity(Entity parent, String name){
		symbols = new HashMap<String, Message>();
		actions = new ArrayList<Action>();
		tempActions = new ArrayList<Action>();
		goalTempActions = new ArrayList<Action>();
		formalParams = new ArrayList<String>();
		actualParams = new ArrayList<Message>();
		tempSender = null;
		tempReceiver = null;
		entityGoals = new ArrayList<Action>();
		goalParameter = new HashMap<Action, Object[]>();
		this.name = name;
		this.parent = parent;
	}
	
	/**
	 * Getter of tempSender.
	 * @return temporary sender
	 */
	public Message getTempSender() {
		return tempSender;
	}

	/**
	 * Setter of tempSender.
	 * @param tempSender temporary sender
	 */
	public void setTempSender(Message tempSender) {
		this.tempSender = tempSender;
	}

	/**
	 * Getter of tempReceiver.
	 * @return temporary receiver
	 */
	public Message getTempReceiver() {
		return tempReceiver;
	}

	/**
	 * Setter of tempReceiver.
	 * @param tempReceiver temporary receiver
	 */
	public void setTempReceiver(Message tempReceiver) {
		this.tempReceiver = tempReceiver;
	}

	public Message getSessionActor() {
		return sessionActor;
	}

	public void setSessionActor(Message sessionActor) {
		this.sessionActor = sessionActor;
	}
	
	/**
	 * Returns a symbol object of the entity that has been declared
	 * or an error if the object is not found.
	 * @param symbol Symbol name
	 * @return Symbol object
	 */
	public Message getSymbol(String symbol){
		if(symbols.containsKey(symbol)) return symbols.get(symbol);
		else if(parent != null) return parent.getSymbol(symbol);
		else { System.out.println("Error - Entity "+getName()+": symbol \""+symbol+"\" not found."); System.exit(3); }
		return null;
	}
	
	/**
	 * Checks if the symbol is contained into the entity symbols data structure.
	 * @param symbol Symbol name
	 * @return true if the entity contains the symbol in the symbols data structure, false otherwise
	 */
	public boolean containsSymbol(String symbol){
		return symbols.containsKey(symbol);
	}
	
	/**
	 * Returns the sender of object of a goal action
	 * or an error.
	 * @param action Goal action
	 * @return Sender object
	 */
	public Message getGoalSender(Action action){
		if(goalParameter.get(action)[0] instanceof Message)
			return (Message) goalParameter.get(action)[0];
		else { System.out.println("Error - The Sender "+ goalParameter.get(action)[0] +" is not an instance of Message"); System.exit(3); }
			return null;
	}
	
	/**
	 * Returns the receiver of object of a goal action
	 * or an error.
	 * @param action Goal action
	 * @return Receiver object
	 */
	public Message getGoalReceiver(Action action){
		if(goalParameter.get(action)[1] instanceof Message)
			return (Message) goalParameter.get(action)[1];
		else { System.out.println("Error - The Receiver "+ goalParameter.get(action)[1] +" is not an instance of Message"); System.exit(3); }
			return null;
	}
	
	/**
	 * Returns the goal label of a goal action
	 * or an error.
	 * @param action Goal action
	 * @return Goal label string
	 */
	public String getGoalLabel(Action action){
		if(goalParameter.get(action)[2] instanceof String)
			return (String) goalParameter.get(action)[2];
		else { System.out.println("Error - The Label "+ goalParameter.get(action)[2] +" is not an instance of String"); System.exit(3); }
			return null;
	}
	
	/**
	 * Returns the goal parameter of a goal action
	 * or an error.
	 * @param action Goal action
	 * @return Goal parameter
	 */
	public Message getGoalArgument(Action action){
		if(goalParameter.get(action)[3] instanceof Message)
			return (Message) goalParameter.get(action)[3];
		else { System.out.println("Error - The argument "+ goalParameter.get(action)[3] +" is not an instance of Message"); System.exit(3); };
			return null;
	}
	
	/**
	 * Adds a symbol to the symbols map of this entity.
	 * @param name Symbol name
	 * @param message Symbol object
	 */
	public void addSymbol(String name, Message message){
		symbols.put(name, message);
	}
	
	/**
	 * Adds a parameter name to the formal parameters of the current entity.
	 * @param name Parameter name.
	 */
	public void addFormalParam(String name){
		formalParams.add(name);
	}
	
	/**
	 * Adds a (goal) action to the entity goals collection.
	 * @param action Goal action
	 */
	public void addToEntityGoals(Action action){
		entityGoals.add(action);
	}
	
	/**
	 * Getter of entityGoals.
	 * @return entityGoals
	 */
	public ArrayList<Action> getEntityGoals(){
		return entityGoals;
	}
	
	/**
	 * Adds a mapping from a (goal) action to the goal parameters in the goalParameter map.
	 * @param action Goal action
	 * @param parameters Parameters object
	 */
	public void addToGoalParameter(Action action, Object[] parameters) { 
		goalParameter.put(action, parameters);
	} 
	
	/**
	 * Getter of formalParams.
	 * @return formalParams
	 */
	public ArrayList<String> getFormalParams() {
		return formalParams;
	}

	/**
	 * Getter of actualParams.
	 * @return actualParams
	 */
	public ArrayList<Message> getActualParams() {
		if(actualParams.size()!=formalParams.size()) {System.out.println("Error - Entity "+getName()+" have not been instantiated"); System.exit(3); }
		return actualParams;
	}
	
	/**
	 * Setter of actualParams
	 * @param actualParams Collection of actual parameters.
	 */
	public void setActualParams(ArrayList<Message> actualParams){
		this.actualParams = actualParams;
	}

	/**
	 * Setter of formalParams
	 * @param formalParams Collection of formal parameter names.
	 */
	public void setFormalParams(ArrayList<String> formalParams) {
		this.formalParams = formalParams;
	}

	/**
	 * Getter of parent.
	 * @return parent
	 */
	public Entity getParent() {
		return parent;
	}
	
	/**
	 * Adds an action to the actions collection.
	 * @param action Action to be added.
	 */
	public void addAction(Action action){
		actions.add(action);
	}
	
//	/**
//	 * Adds an action to the actions collection (at position 0).
//	 * @param action Action to be added.
//	 */
//	public void addActionOnTop(Action action){
//		actions.add(0, action);
//	}
	
	/**
	 * Adds an action to the tempActions collection.
	 * @param action Action to be added.
	 */
	public void addTempAction(Action action){
		tempActions.add(action);
	}
	
	/**
	 * Adds a goal action to the goalTempActions collection.
	 * @param action Action to be added.
	 */
	public void addGoalTempAction(Action action){
		goalTempActions.add(action);
	}
	
	/**
	 * Adds a goal action to the goalTempActions collection (at position 0).
	 * @param action Action to be added.
	 */
	public void addTempActionOnTop(Action action){
		tempActions.add(0, action);
	}
		
	/**
	 * Getter of actions.
	 * @return actions
	 */
	public ArrayList<Action> getActions(){
		return actions;
	}
	
	/**
	 * Getter of tempActions.
	 * @return tempActions
	 */
	public ArrayList<Action> getTempActions(){
		return tempActions;
	}
	
	/**
	 * Getter of goalTempActions.
	 * @return goalTempActions
	 */
	public ArrayList<Action> getGoalTempActions(){
		return goalTempActions;
	}
	
	/**
	 * Getter of symbols.
	 * @return symbols
	 */
	public HashMap<String, Message> getSymbols(){
		return symbols;
	}
	
	/**
	 * Getter of name.
	 * @return name
	 */
	public String getName(){
		return name;
	}
	
	

}
